package com.ruoyi.bst.store.service.impl;

import com.ruoyi.bst.store.domain.StoreQuery;
import com.ruoyi.bst.store.domain.StoreVO;
import com.ruoyi.bst.store.domain.enums.StorePermission;
import com.ruoyi.bst.store.service.StoreService;
import com.ruoyi.bst.store.service.StoreValidator;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.collection.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;

@Service
public class StoreValidatorImpl implements StoreValidator {

    @Autowired
    private StoreService storeService;

    @Override
    public boolean canEdit(Long storeId) {
        return canOperate(Arrays.asList(storeId));
    }

    @Override
    public boolean canDeleteAll(List<Long> storeIds) {
        return canOperate(storeIds);
    }

    @Override
    public boolean canCheckForStore(Long storeId) {
        return canOperate(Arrays.asList(storeId));
    }

    // 是否可以操作店铺
    private boolean canOperate(List<Long> storeIds) {
        if (SecurityUtils.isSysAdmin()) {
            return true;
        }
        if (CollectionUtils.isEmptyElement(storeIds)) {
            return true;
        }
        // 查询店铺
        StoreQuery query = new StoreQuery();
        query.setStoreIds(storeIds);
        query.setScope(true);
        List<StoreVO> StoreVOList = storeService.selectStoreList(query);
        Long userId = SecurityUtils.getUserId();

        // 判断是否可以操作店铺
        for (Long storeId : storeIds) {
            StoreVO store = StoreVOList.stream()
                    .filter(item -> item.getStoreId().equals(storeId))
                    .findFirst().orElse(null);

            if (store == null) {
                return false;
            }
            boolean canOperate = isCreater(store, userId);
            if (!canOperate) {
                return false;
            }
        }
        return true;
    }


    private boolean isCreater(StoreVO store, Long userId) {
        return store != null && store.getUserId() != null && store.getUserId().equals(userId);
    }
}

